/*********************************************************/
/* TAUCS                                                 */
/* Author: Sivan Toledo                                  */
/*********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "taucs.h"

#ifndef TAUCS_CORE
#error "This is a TAUCS core file: you must define a primitive data type"
#endif

#define RNDM ((double)random()/(double)RAND_MAX);

#define max(x,y) ( ((x) > (y)) ? (x) : (y) )

/*********************************************************/
/*                                                       */
/*********************************************************/

#ifdef TAUCS_CORE_DOUBLE
void taucs_vec_permute(int n, int flags, void* v, void* pv, int p[])
{
  if (flags & TAUCS_DOUBLE)
    taucs_dvec_permute(n, (taucs_double*) v, (taucs_double*) pv, p);

  if (flags & TAUCS_SINGLE)
    taucs_svec_permute(n,  (taucs_single*) v, (taucs_single*) pv, p);

  if (flags & TAUCS_DCOMPLEX)
    taucs_zvec_permute(n,  (taucs_dcomplex*) v, (taucs_dcomplex*) pv, p);

  if (flags & TAUCS_SCOMPLEX)
    taucs_cvec_permute(n,  (taucs_scomplex*) v, (taucs_scomplex*) pv, p);
}

void taucs_vec_ipermute(int n, int flags, void* v, void* pv, int p[])
{
  if (flags & TAUCS_DOUBLE)
    taucs_dvec_ipermute(n, (taucs_double*) v, (taucs_double*) pv, p);

  if (flags & TAUCS_SINGLE)
    taucs_svec_ipermute(n,  (taucs_single*) v, (taucs_single*) pv, p);

  if (flags & TAUCS_DCOMPLEX)
    taucs_zvec_ipermute(n,  (taucs_dcomplex*) v, (taucs_dcomplex*) pv, p);

  if (flags & TAUCS_SCOMPLEX)
    taucs_cvec_ipermute(n,  (taucs_scomplex*) v, (taucs_scomplex*) pv, p);
}
#endif

void
taucs_dtl(vec_permute)(int n, taucs_datatype v[], taucs_datatype pv[], int p[])
{
  int i;
  for (i=0; i<n; i++) pv[i] = v[p[i]];
} 

void
taucs_dtl(vec_ipermute)(int n, taucs_datatype pv[], taucs_datatype v[], int invp[]) 
{
  int i;
  for (i=0; i<n; i++) v[invp[i]] = pv[i];
} 

/*********************************************************/
/*                                                       */
/*********************************************************/

